#!/bin/bash
nics()
{
chk=`lsmod | grep $1 | wc --lines`
if [ "$chk" -eq 0 ]
then
	echo "Module" $1 "not present";
	return
fi
echo $1 "nics"
echo "=========="
dev=-1
(echo -e "dev\tnic\tboard\tbdf\tdriver\tmfw\tmac-address\tip-address\tstate\tlink\trdma\tmdump"
echo "--- --- ----- --- ------ --- ------------ ---------- ----- ---- ----------- -----"
for nic in `for i in \`ls /sys/bus/pci/drivers/$1/\`; do ls /sys/bus/pci/devices/$i/net 2> /dev/null; pci=$i; done`;
do
	dev=$[$dev + 1]
	bdf=`ethtool -i $nic | grep bus | grep "[0-9a-f][0-9a-f]:[0-9a-f][0-9a-f]\.[0-9a-f]" -o`
	if [ -z $bdf ]
	then
		bdf="-"
	fi
	board=`lspci -s $bdf -vv 2> /dev/null | grep -m 1 -P '(BCM|QL)[0-9]\S*' -o`
	if [ -z $board ]
	then
		board="-"
	fi
	drv_ver=`ethtool -i $nic | grep ^version | cut -d" " -f2`
	if [ -z $drv_ver ]
	then
		drv_ver="-"
	fi
	bc_ver=`ethtool -i $nic | grep firmware | grep "mfw [0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d " " -f2`
	if [ -z $bc_ver ]
	then
		bc_ver=`ethtool -i $nic | grep firmware | grep "[0-9].[0-9].[0-9].[0-9]" | cut -d" " -f2`
		if [ -z $bc_ver ]
		then
			bc_ver="-"
		fi
	fi
	mac=`ip -o link show $nic | grep -o ..:..:..:..:..:.. | head -n 1`
	if [ -z $mac ]
	then
		mac="-"
	fi

	state=`ip -o link show $nic | grep ',UP' -o | grep 'UP' -o`
	if [ -z $state ]
	then
		state="-"
	fi

	vf="-(PF`ethtool -i $nic | grep bus | grep .$ -o`)\t"
	if [ -z $vf ]
	then
		vf="-"
	fi
	if [ `lspci -s $bdf | grep Virtual -o` ]
	then
		physfn=/sys/bus/pci/devices/0000:$bdf/physfn
		for virtfn in `ls $physfn | grep virtfn`
		do
			ls $physfn/$virtfn/net/$nic &> /dev/null && vf="-(VF`echo $virtfn | grep "[0-9]*" -o`_of_PF`ethtool -i \`ls $physfn/net\` | grep bus | grep .$ -o`)"
		done
	fi
	link=`ethtool $nic | grep "Link detected" | grep "yes\|no" -o`
	if [ -z $link ]
	then
		link="-"
	fi

	ip="`ip -o -4 a s $nic | grep -o '[0-9]*.[0-9]*.[0-9]*.[0-9]*/[0-9]*'`  "
	if [ "$ip" == "  " ]; then ip=" - \t\t"; fi

	rdma=`ls -d /sys/class/infiniband/*/device/net/$nic 2> /dev/null| cut -d'/' -f5`;
	if [ -z $rdma ]
	then
		rdma="-"
	else
		rdma_type=`cat /sys/class/infiniband/$rdma/node_type | cut -f1 -d":"`
		if [[ ${rdma_type} == 1 ]]
		then
			rdma="$rdma,RoCE"
		elif [[ ${rdma_type} == 4 ]]
		then
                        rdma="$rdma,iWARP"
		else
			rdma="$rdma,$rdma_type"
		fi
	fi
	
	mdump="-"
	if [[ -f /sys/kernel/debug/qed/$bdf/mdump ]]
	then
		echo status > /sys/kernel/debug/qed/$bdf/mdump
		if [[ $? == 0 ]]
		then 
			num_dump=`cat /sys/kernel/debug/qed/$bdf/mdump | grep --text num_of_logs | cut -d " " -f2 | head -n 1`
			if [[ ! -z $num_dump && $num_dump != 0 ]]
			then
				mdump="yes"
			fi
		fi
	fi

	if [[ "$ip" == " - \t\t" ]] 
	then
		echo -e "$dev\t$nic$vf\t$board\t$bdf\t\t$drv_ver\t$bc_ver\t\t$mac\t$ip\t$state\t$link\t$rdma\t$mdump"

	else	
		ip_count=0;
		for i in $ip; do
			if [[ $ip_count == 0 ]]
			then
				echo -e "$dev\t$nic$vf\t$board\t$bdf\t\t$drv_ver\t$bc_ver\t\t$mac\t$i\t$state\t$link\t$rdma\t$mdump"
			else
				echo -e "-\t-\t-\t-\t-\t-\t-\t$i\t-\t-\t-"
			fi
		ip_count=$((ip_count + 1))
		done
	fi
	vlans_if=`ip -o link show | grep $nic | grep @ | cut -d " " -f 2 | cut -d "@" -f 1`
	for vlan in $vlans_if; do
		mac=`ip -o link show $vlan | grep -o ..:..:..:..:..:.. | head -n 1`
		if [ -z $mac ]
		then
			mac="-"
		fi
		state=`ip -o link show $vlan | grep ',UP' -o | grep 'UP' -o`
		if [ -z $state ]
		then
			state="-"
		fi
		link=`ethtool $vlan | grep "Link detected" | grep "yes\|no" -o`
		if [ -z $link ]
		then
			link="-"
		fi

		ip="`ip -o -4 a s $vlan | grep -o '[0-9]*.[0-9]*.[0-9]*.[0-9]*/[0-9]*'`  "
		if [ "$ip" == "  " ]; then ip=" - \t\t"; fi

		if [[ "$ip" == " - \t\t" ]] 
		then
			echo -e "-\t$vlan\t-\t-\t-\t-\t-\t$ip\t$state\t$link\t-\t-"

		else
			ip_count=0;
			for i in $ip; do
				if [[ $ip_count == 0 ]]
				then
					echo -e "-\t$vlan\t-\t-\t-\t-\t-\t$i\t$state\t$link\t-\t-"
				else
					echo -e "-\t-\t-\t-\t-\t-\t-\t$i\t-\t-\t-\t-"
				fi
			ip_count=$((ip_count + 1))
			done
		fi
	done	
		
done
) |column -t
}

if [ -z $1 ]
then
	nics bnx2x
	echo ""
	nics qede
else
	nics $1
fi

